'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const ServiceReconciler_1 = require("../service/ServiceReconciler");
const pandora_env_1 = require("pandora-env");
const ProcessContextAccessor_1 = require("./ProcessContextAccessor");
const pandora_hub_1 = require("pandora-hub");
const LoggerBroker_1 = require("../universal/LoggerBroker");
/**
 * Class ProcessContext
 *  1. Inject the service
 *  2. manage the process's lifecycle, provide start() and stop()
 *  3. drive the lifecycle of the service
 */
class ProcessContext {
    constructor(processRepresentation) {
        this.processRepresentation = processRepresentation;
        this.serviceReconciler = new ServiceReconciler_1.ServiceReconciler(processRepresentation, this);
        this.processContextAccessor = new ProcessContextAccessor_1.ProcessContextAccessor(this);
    }
    /**
     * Get environment object
     * @returns {Environment}
     */
    getEnvironment() {
        return pandora_env_1.EnvironmentUtil.getInstance().getCurrentEnvironment();
    }
    getIPCHub() {
        if (!this.ipcHub) {
            this.ipcHub = new pandora_hub_1.Facade();
            this.ipcHub.setup({
                location: {
                    appName: this.processRepresentation.appName,
                    processName: this.processRepresentation.processName,
                    pid: process.pid.toString()
                },
                logger: LoggerBroker_1.consoleLogger
            });
        }
        return this.ipcHub;
    }
    /**
     * Bind services
     * @param {ServiceRepresentation | ServiceRepresentation[]} services
     */
    bindService(services) {
        if (Array.isArray(services)) {
            for (const service of services) {
                this.serviceReconciler.receiveServiceRepresentation(service);
            }
            return;
        }
        this.serviceReconciler.receiveServiceRepresentation(services);
    }
    /**
     * Start worker, drive the lifecycle of the service
     * @returns {Promise<void>}
     */
    async start() {
        if (!process.env.SKIP_IPC_HUB) {
            await this.getIPCHub().start();
        }
        await this.serviceReconciler.start();
    }
    /**
     * Stop worker, drive the lifecycle of the service
     * @returns {Promise<void>}
     */
    async stop() {
        if (!process.env.SKIP_IPC_HUB) {
            await this.getIPCHub().stop();
        }
        await this.serviceReconciler.stop();
    }
}
exports.ProcessContext = ProcessContext;
//# sourceMappingURL=ProcessContext.js.map